#!/usr/bin/env python

"""
FUNCTION: Uses the database to create files describen the color code of
          tree nodes in iTOL.

USAGE: NHase_db_create_iTOL_color_files.py <DB_FILE> <DOMAIN>

Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import sys

import nhase_db_tools

def main():
    if not len(sys.argv) == 3:
        print __doc__
        sys.exit(2)

    creator = ITOLColorFileCreator(sys.argv[1], sys.argv[2])
    creator.get_domain_HMM_hit_names_and_simp_headers()
    creator.create_taxonomy_color_file()
    creator.create_protein_function_color_file()
    creator.create_environments_color_file()
    creator.create_bitscore_color_file()
    creator.create_hit_length_color_file()

class ITOLColorFileCreator(object):

    output_file_suffix = 'iTOL_color'
    annotation_group_to_color_code = {
        'NHase' : '#FFCF99', 
        'Putative_NHase' : '#FCFF99', 
        'Thiocyanate_hydrolase' : '#F4D1FF',
        'Unknown' : '#E6E6E6',
        }

    environment_to_color_code = {
        'GOS' : '#66A3FF',
        'MFS' : '#FFC105', 
        'WLF' : '#99EBFF', 
        'EBPRS' : '#B88A00', 
        'HGUT' : '#FFD4A8', 
        'NPSG' : '#8AB800', 
        'AMD' : '#FF6694', 
        }

    phyla_to_color = {
        'Actinobacteria' : '#D2A679',
        'Bacteroidetes/Chlorobi' : '#6BDAFF',
        'Bacillus/Clostridium' : '#FFD24D',
        'Choanoflagellida' : '#ADD4AA',
        'Cyanobacteria' : '#CCE6FF', 
        'Firmicutes' : '#8888C3', 
        'Proteobacteria' : '#FF8080',
        'uncultured' : '#D9D9D9',
        }

    def __init__(self, db_file, domain):
        self.db_connector = nhase_db_tools.NHasesDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.domain = domain

    def create_protein_function_color_file(self):
        """Creates an iTOL color file to colorize the node ring based
        on the functional groups that determined by anaylizing the
        protein annotation string. Only done for UniRef and STRING
        proteins.

        """
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.protein_function' % (
            self.domain, self.output_file_suffix), 'w')
        
        for domain_HMM_hit_name, hit_protein_name, simplified_headers in (
            self._get_data_triples(cur)):
            annotation = self.db_connector.get_annotation_string_of_protein(
                hit_protein_name)

            if annotation == None: annotation_group = '-'
            source = self.db_connector.get_source_of_protein(hit_protein_name)
            # Skip environments here
            if source not in ['UniRef']:
                continue
            # UniRef and String
            else:
                annotation_group = (
                    self.db_connector.get_text_string_annotation_group_of_protein(
                    hit_protein_name))

            if not annotation_group == '-':
                color = self._get_function_color(annotation_group)
                output_file.write('%s\t%s\t%s\n' % (
                    simplified_headers,
                    color,
                    annotation_group))
        output_file.close()
        self.db_connector.close_db_connection()

    def create_taxonomy_color_file(self):
        """Creates an iTOL color file to colorize the node ring based
        on the taxonomic group of the species.
        """
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.taxonomic_groups' % (
            self.domain, self.output_file_suffix), 'w')
        for domain_HMM_hit_name, hit_protein_name, simplified_headers in (
            self._get_data_triples(cur)):
            species = self.db_connector.get_species_of_protein(hit_protein_name)
            tax_string = self.db_connector.get_full_tax_of_species(species)
            tax_group = '-'
            color = '#0000000'
            if not tax_string == None:
                tax_group = self._get_phylo_group_from_tax_string(tax_string)
            # Fix some special cases explicitly
            elif not species == '-':
                if species == 'Labrenzia aggregata':
                    tax_group = 'Proteobacteria'
                elif species == 'Rhizobium loti':
                    tax_group = 'Proteobacteria'
                elif species == 'Rhizobium meliloti':
                    tax_group = 'Proteobacteria'
                elif species == 'uncultured bacterium':
                    tax_group = 'uncultured'
                    
            if not tax_group == '-':
                color = self._get_taxonomy_color(tax_group)
                output_file.write ("%s\t%s\t%s (%s)\n" % (
                    simplified_headers, color, tax_group, species))
        output_file.close()          
        self.db_connector.close_db_connection()

    def create_environments_color_file(self):
        """Creates an iTOL color file to colorize the node ring based
        on the environment a protein hit is coming from.

        """
        
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.environment' % (
            self.domain, self.output_file_suffix), 'w')

        for domain_HMM_hit_name, hit_protein_name, simplified_headers in (
            self._get_data_triples(cur)):
            source = self.db_connector.get_source_of_protein(hit_protein_name)
            if source == None or source == 'UniRef': continue
            if self.environment_to_color_code.has_key(source):
                output_file.write ("%s\t%s\n" % (
                    simplified_headers,
                    self.environment_to_color_code[source]))
            else:
                print 'No color code for "%s"!' % source
        output_file.close()
        self.db_connector.close_db_connection()

    def create_bitscore_color_file(self):
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.bitscores' % (
            self.domain, self.output_file_suffix), 'w')
        min_bitscore = 0
        max_bitscore = 0
        simplified_headers_and_bitscores = []
        # Get ids, bitscore and find the max and min bitscore
        for domain_HMM_hit_name, hit_protein_name, simplified_headers in \
            self._get_data_triples(cur):
            hit_features = self.db_connector.get_hit_features(
                domain_HMM_hit_name)
            bitscore = float(hit_features['bitscore'])
            simplified_headers_and_bitscores.append(
                [simplified_headers, bitscore])
            if bitscore < min_bitscore:
                min_bitscore = bitscore
            if bitscore > max_bitscore:
                max_bitscore = bitscore
        # Calculate the normalized bitscore * color value
        for simplified_headers, bitscore in simplified_headers_and_bitscores:
            normalized_bitscore = 0
            if min_bitscore >= 0:
                try:
                    normalized_bitscore = (
                        (bitscore - min_bitscore) / (max_bitscore - min_bitscore))
                except:
                    normalized_bitscore = 0
            elif min_bitscore < 0:
                try:
                    normalized_bitscore = (
                        (bitscore + abs(min_bitscore)) /
                        (max_bitscore + abs(min_bitscore)))
                except:
                    normalized_bitscore = 0
            color_code = None
            if normalized_bitscore != 0:
                color_code = int(255 - normalized_bitscore * 255)
            else:
                color_code = int(255)
            hex_color_code = '#%02x%02x%02x' % (
                color_code, color_code, color_code)
            output_file.write (
                "%s\t%s\tbitscore: %s, normalized bitscrore: %s\n" % (
                simplified_headers, hex_color_code, bitscore, normalized_bitscore))

    def create_hit_length_color_file(self):
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        cur = self.con.cursor()
        output_file = open('%s_%s.hit_length' % (
            self.domain, self.output_file_suffix), 'w')
        hit_lengths = []
        simplified_headers_and_hit_lengths = []
        # Get ids, hit postioin and find the max and min hit length
        for domain_HMM_hit_name, hit_protein_name, simplified_headers in \
            self._get_data_triples(cur):
            hit_features = self.db_connector.get_hit_features(
                domain_HMM_hit_name)
            hit_length = (float(hit_features['hit_end_position']) -
                          float(hit_features['hit_start_position']))
            simplified_headers_and_hit_lengths.append(
                [simplified_headers, hit_length])
            hit_lengths.append(hit_length)
        hit_lengths.sort()
        min_hit_length = hit_lengths[0]
        max_hit_length = hit_lengths[-1]
        # Calculate the normalized hit_length * color value
        for simplified_headers, hit_length in simplified_headers_and_hit_lengths:
            normalized_hit_length = 0
            if min_hit_length >= 0:
                try:
                    normalized_hit_length = (
                        (hit_length - min_hit_length) / (
                        max_hit_length - min_hit_length))
                except:
                    normalized_hit_length = 0
            elif min_hit_length < 0:
                try:
                    normalized_hit_length = (
                        (hit_length + abs(min_hit_length)) /
                        (max_hit_length + abs(min_hit_length)))
                except:
                    normalized_hit_length = 0
            color_code = None
            if normalized_hit_length != 0:
                color_code = int(255 - normalized_hit_length * 255)
            else:
                color_code = int(255)
            hex_color_code = '#%02x%02x%02x' % (
                color_code, color_code, color_code)
            output_file.write (
                "%s\t%s\thit length: %s, normalized hit length: %s\n" % (
                simplified_headers, hex_color_code, int(hit_length),
                normalized_hit_length))

                    
    def _protein_annotation_score_to_color_code(self, score):
        color_code = '#CCCCCC'
        if score == None:
            return(color_code)
        if score > 0:
            red_value_dec = int(score * 255)
            # If the value goes too low the color would become to dark
            if red_value_dec < 120: red_value_dec = 100
            # Transform to hex
            red_value_hex = str(hex(red_value_dec))[-2:]
            color_code = '#%s0000' % red_value_hex
        elif score < 0:
            green_value_dec = int(-score * 255)
            # If the value goes too low the color would become to dark
            if green_value_dec < 120: red_value_dec = 100
            # Transform to hex
            green_value_hex = str(hex(green_value_dec))[-2:]
            color_code = '#00%s00' % green_value_hex
        return(color_code)

    def get_domain_HMM_hit_names_and_simp_headers(self):
        self.db_connector.create_db_connection()
        self.con = self.db_connector.get_connection()
        self.domain_HMM_hit_names_and_simp_headers = {}
        cur = self.con.cursor()
        cur.execute('SELECT domain_HMM_hit_name, simplified_hit_header '
                    'FROM HMM_hits WHERE HMM_domain_name="%s";' % (
            self.domain))
        for domain_HMM_hit_name, simplified_header in cur.fetchall():
            # Discard entries that are added as cluster members; they
            # are not found in the tree.
            if simplified_header == '-': continue
            self.domain_HMM_hit_names_and_simp_headers[
                domain_HMM_hit_name] = simplified_header
        self.db_connector.close_db_connection()

    def _get_data_triples(self, cur):
        for domain_HMM_hit_name in \
                self.domain_HMM_hit_names_and_simp_headers.keys():
            # Get protein name of the hits
            cur.execute('SELECT hit_protein_name FROM HMM_hits '
                        'WHERE domain_HMM_hit_name="%s";' % (
                domain_HMM_hit_name))
            hit_protein_name = cur.fetchone()[0]
            yield(domain_HMM_hit_name, hit_protein_name,
                  self.domain_HMM_hit_names_and_simp_headers[
                domain_HMM_hit_name])

    def _get_function_color(self, annotation_group):
        return(self.annotation_group_to_color_code[annotation_group])

    def _get_taxonomy_color(self, tax_group):
        return(self.phyla_to_color[tax_group])

    def _get_phylo_group_from_tax_string(self, tax_string):
        level_name_group_triples  = [
            [3, 'Proteobacteria', 'Proteobacteria'],
            [3, 'Cyanobacteria', 'Cyanobacteria'],
            [3, 'Bacteroidetes/Chlorobi group', 'Bacteroidetes/Chlorobi'],
            [3, 'Alveolata', 'Alveolata'],
            [3, 'Bacillus/Clostridium group', 'Bacillus/Clostridium'],
            [3, 'Actinobacteria', 'Actinobacteria'],
            [3, 'Deinococcus-Thermus', 'Deinococcus-Thermus'],
            [3, 'Planctomycetes', 'Planctomycetes'],
            [3, 'Aquificae', 'Aquificae'],
            [3, 'Chromophyta', 'Chromophyta'],
            [3, 'Chloroflexi', 'Chloroflexi'],
            [3, 'Mycetozoa', 'Mycetozoa'],
            [3, 'Spirochaetes', 'Spirochaetes'],
            [3, 'Fibrobacter/Acidobacteria group', 'Fibrobacter/Acidobacteria'],
            [3, 'Fusobacteria', 'Fusobacteria'],
            [3, 'Chlamydiae/Verrucomicrobia group',
             'Chlamydiae/Verrucomicrobia'],
            [3, 'Firmicutes', 'Firmicutes'],
            [3, 'Chlorobionta', 'Viridiplantae'],
            [3, 'Viridiplantae', 'Viridiplantae'],
            [1, 'Viruses', 'Viruses'],
            [1, 'other sequences', 'Others'],
            [4, 'Fungi', 'Fungi'],
            [4, 'Choanoflagellida', 'Choanoflagellida'],
            [3, 'Not yet classified (Prokaryotae)', 'Unclassified Prokaryotae'],
            [4, 'Animalia', 'Animal'],
            [4, 'Metazoa', 'Animal'],
            [2, 'Archaea', 'Archaea'],
            [3, 'Euglenozoa', 'Euglenozoa'],
            [3, 'Mycetozoa', 'Mycetozoa'],
            [3, 'unclassified eukaryotes', 'Others_eukaryotes'],
            [3, 'unclassified eukaryotes', 'Others_eukaryotes'],
            [3, 'stramenopiles', 'Others_eukaryotes'],
            [3, 'Cercozoa', 'Others_eukaryotes'],
            [3, 'Haptophyceae', 'Others_eukaryotes'],
            [3, 'Cryptophyta', 'Others_eukaryotes'],
            [2, 'Bacteria', 'Others_bacteria'],
            [1, 'unclassified sequences', 'Others'],
            ]
        split_tax_string = tax_string.split('|')
        taxonomy_group = None

        for taxonomy_level, name, resulting_group in level_name_group_triples:
            if taxonomy_level > len(split_tax_string) - 1 : continue
            if split_tax_string[taxonomy_level] == name:
                taxonomy_group = resulting_group
                break
            
        if not taxonomy_group == None:
            return(taxonomy_group)
        else:
            if not len(tax_string) == 0:
                print 'No fitting taxonomy group found for "%s". '\
                      'Please adapt code.' % (tax_string)
            return('-')

if __name__ == '__main__': main()
