"""
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2007 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

class RegionInSequenceMappingParser:
    """Parses files containg regions

    (ORF - complete or incomplete) mappings to sequences (contigs,
    singlets or reads). These input file can produce for example by
    Christian von Merings 'make_regions_per_contig_summary.pl' .
    
    """
    def give_region_to_sequence_mappings(self, input_file_name):
        """Returns the values of each entry. Implemented as a generator."""
        input_file = open(input_file_name)
        for line in input_file.xreadlines():
            if line[0] in ['#', '\n']:
                continue
            split_line = line[:-1].split()
            yield {
                'sequence_name' : split_line[0],
                'region_name' :  split_line[1],
                'region_start' :  int(split_line[2]),
                'region_end' :  int(split_line[3]),
                'region_direction' :  split_line[4]}
        input_file.close()

class COGInSequenceMappingParser:
    """Parse COG to sequence mapping files

    There files are produced by Christian von Merings
    'map_orthgroups_secondary.pl'.

    """
    def give_COG_to_sequence_mapping(self, input_file_name):
        input_file = open(input_file_name)
        for line in input_file.xreadlines():
            if line[0] in ['#', '\n']:
                continue
            split_line = line[:-1].split()
            yield {
                'sequence_name' : split_line[0],
                'region_start' :  int(split_line[2]),
                'region_end' :  int(split_line[3]),
                'region_direction' :  split_line[4],
                'COG_name' :  split_line[5],
                'bitscore' :  float(split_line[6]),
                'quality' :  split_line[7]}
        input_file.close()

class MFSNameTranslator:
    """Helps to solve the name of the Minnesota Farm Soil sequence

    The namen in different file of Minnesota Farm Soil are different.
    This confusion is solved by using this class.

    e.g:
    XZS29930_x1 (in region mapping file) and
    XZS29930.x1 (in the fasta file)

    Contig42728 (in region mapping file) and
    2662324_fasta.screen.Contig42728 (in the fasta file)

    """
    def translate_region_mapping_name_to_fasta_name(self, seq_name):
        if seq_name.startswith('XZS'):
            seq_name = seq_name.replace('_', '.')
        elif seq_name.startswith('Contig'):
            seq_name = '2662324_fasta.screen.%s' % seq_name
        return seq_name
        

class FastaHeaderToEnvTranslator(object):
    """Uses the first letter of a fasta header to determine the source
    environment

    """

    def translate_fasta_header_to_environment(self, header):
        source = 'unknown'
        # String
        if len(header.split('.')) == 2 and \
                 header[0] in ['0', '1', '2', '3', '4',
                                     '5', '6', '7', '8', '9']:
            source = 'String'
        # Saragosso Sea sample
        elif header.startswith('AAC'):
            source = 'SGS'
        # Minnesota Fram Soil
        elif header.startswith('Contig') or \
                 header.startswith('XZS'):
            source = 'MFS'
        # Whalefall
        elif header.startswith('AHAA') or \
            header.startswith('AHAI') or \
            header.startswith('AGZO'):
            source = 'WLF'
        # Human Gut
        elif header.startswith('20040'):
            source = 'HGUT'
        # Acid Mine drainage
        elif header.startswith('AMD') or \
                 header.startswith('FERII') or \
                 header.startswith('FerI') or \
                 header.startswith('Gp1_scaffol') or \
                 header.startswith('LeptoIII') or \
                 header.startswith('LeptoII'):
            source = 'AMD'
        # Enhanced Biological Phosphorus Removal Sludges
        elif header.startswith('20000') or \
                 header.startswith('20001') or \
                 header.startswith('20002') or \
                 header.startswith('20003') or \
                 header.startswith('20004') or \
                 header.startswith('20005') or \
                 header.startswith('20006') or \
                 header.startswith('20010') or \
                 header.startswith('20011'):
            source = 'EBPRS'
        # Global Ocean Sampling
        elif header.startswith('JCVI_PEP_'):
            source = 'GOS'
        # Eel River Basin Sediment
        elif header.startswith('PNS') or \
                 header.startswith('TIF') or \
                 header.startswith('XXV'):
            source = 'ERBS'
        # North Pacific Subtropical Gyre
        elif header.startswith('DU7') or \
                 header.startswith('DU8'):
            source = 'NPSG'
        return(source)

        
