#!/usr/bin/env python
"""
FUNCTION: Adds features of HMM hit proteins to the database and reads
          one or more fasta files of the proteins searched for this.

USAGE: NHase_db_add_protein_properties.py <DB_FILE> <FASTA_FILE_1> [<FASTA_FILE_2>]  <DB_SWITCH>

       <DB_SWITCH> can be
         - 'insert' to put new data into the database
         - 'update' to change entries in the database
         
Copyright (c) 2008, Konrad Foerstner <konrad@foerstner.org>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
         
"""
__author__ = "Konrad Foerstner <konrad@foerstner.org>"
__copyright__ = "2008 by Konrad Foerstner <konrad@foerstner.org>"
__license__ = "ISC license"

import os
import sys

import fastaparser
import nhase_db_tools
import metagenomicseq

def main():
    if len(sys.argv) < 4 or (not sys.argv[-1] in ['insert', 'update']):
        print __doc__
        sys.exit(2)
    adder = ProteinFeatureAdder(sys.argv[1])
    adder.read_fasta_file(sys.argv[2:-1], sys.argv[-1])

class ProteinFeatureAdder(object):

    def __init__(self, db_file):
        self.db_connector = nhase_db_tools.NHasesDBConnector(db_file)
        if not self.db_connector.check_db_file() == True:
            print 'The database file "%s" does not exist!' % (db_file)
            sys.exit(2)
        self.fasta_parser = fastaparser.FastaParser()
        self.env_translator = metagenomicseq.FastaHeaderToEnvTranslator()
        
    def read_fasta_file(self, fasta_files, insert_update_switch):
        self.__check_file_existence(fasta_files)
        self.db_connector.create_db_connection()
        con = self.db_connector.get_connection()
        counter = 0
        for fasta_file in fasta_files:
            for header, seq in self.fasta_parser.parse_fasta_file(fasta_file):
                sys.stdout.write(str(counter))
                sys.stdout.write("\r")
                sys.stdout.flush()
                counter += 1
                protein_features = self.__get_protein_features(header, seq)
                short_header = header.split()[0]
                cur = con.cursor()
                cur.execute('SELECT hit_protein_name FROM HMM_hits WHERE '
                            'hit_protein_name="%s";' % (short_header))
                if not cur.fetchone() == None:
                    protein_features = self.__get_protein_features(header, seq)
                    if insert_update_switch == 'insert':
                        self.db_connector.fill_protein_table(protein_features)
                    elif insert_update_switch == 'update':
                        self.db_connector.update_protein_table(protein_features)
        self.db_connector.close_db_connection()

    def __check_file_existence(self, fasta_files):
        for fasta_file in fasta_files:
            if os.path.exists(fasta_file) == False:
                print 'Error: File "%s" does not exist.' % (fasta_file)
                sys.exit(2)
                
    def __get_protein_features(self, header, seq):
        protein_name = header.split()[0]
        protein_length = len(seq)
        annotation = self._clean_annotation(' '.join(header.split()[1:]))
        species = '-'
        source = '-'
        if header.startswith('UniRef'):
            species = ' '.join(annotation.split(' - ')[-1].split()[0:2])
            source = 'UniRef'
        else:
            source = (
                self.env_translator.translate_fasta_header_to_environment((
                protein_name)))
            if source == 'String':
                species = annotation.split('(')[1].split(')')[0]
        return({'protein_name' : protein_name,
                'protein_length' : protein_length,
                'annotation' : annotation,
                'source' : source,
                'species' : species})    

    def _clean_annotation(self, annotation):
        annotation = annotation.replace('"', '')
        annotation = annotation.replace("'", '')
        return(annotation)

if __name__ == '__main__': main()
